/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.model.iface;

import java.util.List;

import javax.xml.namespace.QName;

import com.eviware.soapui.impl.wsdl.support.soap.SoapVersion;
import com.eviware.soapui.impl.wsdl.support.wsdl.WsdlContext;
import com.eviware.soapui.impl.wsdl.support.wsdl.WsdlLoader;
import com.eviware.soapui.model.ModelItem;
import com.eviware.soapui.model.project.Project;

/**
 * An Interface exposing operations
 * 
 * @author Ole.Matzura
 */

public interface Interface extends ModelItem
{
	/**
	 * property for endpoint changes, propertychange messages will contain oldValue = null if an
	 * endpoint is added, newValue = null if an endpoint is removed and both values if an endpoint is
	 * modified.
	 */
	
   public final static String ENDPOINT_PROPERTY = Interface.class.getName() + "@endpoint";
   
   /**
    * property for definition changes
    */
   
   public final static String DEFINITION_PROPERTY = Interface.class.getName() + "@definition";  

   /**
    * property for isUpdating() changes
    */
   public final static String UPDATING_PROPERTY = Interface.class.getName() + "@updating";  
   
   public String [] getEndpoints();
   
   public void addEndpoint( String endpoint );
   
   public void removeEndpoint( String endpoint );
   
   public void changeEndpoint( String oldEndpoint, String newEndpoint );

   public Operation getOperationAt(int index);

   public int getOperationCount();

   public Operation getOperationByName(String name);
   
   public Project getProject();
   
   public String getDefinition();
   
   public SoapVersion getSoapVersion();
   
   public void addInterfaceListener( InterfaceListener listener );
   
   public void removeInterfaceListener( InterfaceListener listener );
   
   public MessageBuilder getMessageBuilder();
   
   /**
    * This shouldn't be here or refactored to an InterfaceContext interface.. 
    */
   
   public WsdlContext getWsdlContext();

   public QName getBindingName();

   public boolean isCached();

   public WsdlLoader createWsdlLoader();

	public List<Operation> getOperationList();
	
	public boolean isUpdating();
}
